import unittest

from .test_api import TestDesigner, TestConnection, TestEnterprise, TestClusterMod, TestSessionMod, TestInfobaseMod
from .test_utils import TestUtils, TestPlatform

__all__ = [
    'TestDesigner',
    'TestEnterprise',
    'TestConnection',
    'TestUtils',
    'TestPlatform',
    'TestSessionMod',
    'TestInfobaseMod',
    'TestClusterMod'
]

if __name__ == '__main__':
    unittest.main()



import logging
from designer_cmd.utils import PlatformVersion, get_rac_path, execute_command
from typing import List, Dict, Optional
from abc import ABC
from enum import Enum
from dataclasses import dataclass, field


logger = logging.getLogger(__name__)


@dataclass
class SqlServerConnection:
    host: str
    user: str
    password: str
    type: 'SqlServerType'
    port: int = field(default=5432)

    @property
    def server_addr(self):
        return f'{self.host}:{self.port}'


class SqlServerType(Enum):
    MSSQL = 'MSSQLServer'
    POSTGRE_SQL = 'PostgreSQL'
    IBMDB2 = 'IBMDB2'
    DB2 = 'DB2'
    ORACLE = 'OracleDatabase'


class RacConnection:

    def __init__(self, user: str = '',
                 password: str = '',
                 server: str = '',
                 port: int = 1545,
                 base_user: str = '',
                 base_password: str = '',
                 timeout: int = 10):
        self.server = server
        self.port = port
        self.user = user
        self.password = password
        self.base_user = base_user
        self.base_password = base_password
        self.timeout = timeout

    def get_connection_string(self):
        return f'{self.server}:{self.port}'

    def get_credentials(self):
        cred = []
        if self.user:
            cred.append(f'--cluster-user={self.user}')
        if self.password:
            cred.append(f'--cluster-pwd={self.password}')
        return cred

    def get_base_credentials(self):
        cred = []
        if self.base_user:
            cred.append(f'--infobase-user={self.base_user}')
        if self.base_password:
            cred.append(f'--infobase-pwd={self.base_password}')
        return cred

    def __str__(self):
        attrs = [f'server: {self.server}', f'port: {self.port}']
        if self.user:
            attrs.append(f'cluster_user: {self.user}')
        if self.base_user:
            attrs.append(f'base_user: {self.base_user}')
        attr_str = ' '.join(attrs)
        return f'<RacConnection> {attr_str}'


def parse_result(result_str: str) -> List[Dict[str, str]]:
    result = []

    try:
        b_data = result_str.encode('cp866')
        result_str = b_data.decode('utf-8')
    except UnicodeEncodeError:
        pass

    data = result_str.split('\n')
    cur_result = {}
    for el in data:
        if el == '':
            if cur_result:
                result.append(cur_result)
            cur_result = {}
            continue

        key_val = el.split(':')
        if len(key_val) < 2:
            raise ValueError('Ошибка разбора результата ответа')
        key = key_val[0].strip()
        val = ':'.join(key_val[1:]).strip()
        cur_result[key] = val
    if cur_result:
        result.append(cur_result)
    return result


def required_cluster_id(func):

    def warper(self: "ABCRacMod", *args, **kwargs):
        if self.executor.custer_id is None:
            self.executor.set_cluster_id()
        return func(self, *args, **kwargs)

    return warper


def required_base_id(func):

    def warper(self: "ABCRacMod", *args, **kwargs):
        if self.executor.base_id is None:
            raise AttributeError('base_id not set')
        return func(self, *args, **kwargs)

    return warper


class Rac:

    def __init__(self, platform_version: str, connection: RacConnection):
        self.platform_version: PlatformVersion = PlatformVersion(platform_version)
        self.connection = connection
        self.executable_path = get_rac_path(self.platform_version)

        self._cluster_id = None
        self._base_id = None

        self.infobase = InfobaseMod(self)
        self.cluster = ClusterMod(self)
        self.sessions = SessionMod(self)

    def add_cluster_id(self, params: list, cluster_id: Optional[str] = None):
        if not cluster_id:
            cluster_id = self._cluster_id
        if not cluster_id:
            raise ValueError('Cluster_id не установлен!')
        params.append(f'--cluster={cluster_id}')

    def add_base_id(self, params: list, base_id: Optional[str] = None):
        if not base_id:
            base_id = self._base_id
        if not base_id:
            raise ValueError('base_id не установлен!')
        params.append(f'--infobase={base_id}')

    @property
    def custer_id(self):
        return self._cluster_id

    def set_cluster_id(self, cluster_id: str = None):
        if not cluster_id:
            cluster_data = self.cluster.get_cluster_data()
            cluster_id = cluster_data.get('cluster')
        self._cluster_id = cluster_id

    @property
    def base_id(self) -> str:
        return self._base_id

    @base_id.setter
    def base_id(self, value: str):
        self._base_id = value

    def execute_command(self, mode: str, command_params: list) -> List[Dict[str, str]]:
        params = [self.connection.get_connection_string(), mode]

        params += self.connection.get_credentials()

        params += command_params

        str_command = ' '.join(params)

        logger.debug(f'Выполняю команду {self.executable_path} {str_command}')

        result = execute_command(self.executable_path, params, self.connection.timeout)

        if result[0] == 0:
            result_data = parse_result(result[1])
        else:
            raise SyntaxError(f'Не удалось выполнить команду! подробно: {result[1]}')
        return result_data

    def disconnect_users(self, base_ref: str):
        base_data = self.infobase.get_base_by_ref(base_ref)
        base_id = base_data.get('infobase')
        self.base_id = base_id

        session_list = self.sessions.get_session_list()
        for session in session_list:
            self.sessions.terminate_session(session.get('session'))


class ABCRacMod(ABC):

    def __init__(self, executor: Rac, mod: str):
        self.executor = executor
        self.mod = mod

    def execute_command(self, command_params: list) -> List[Dict[str, str]]:
        return self.executor.execute_command(self.mod, command_params)


class InfobaseMod(ABCRacMod):

    def __init__(self, executor: Rac):
        super(InfobaseMod, self).__init__(executor, 'infobase')

    def execute_command(self, command_params: list,
                        base_id_required: bool = True,
                        base_cred_required: bool = False) -> List[Dict[str, str]]:
        self.executor.add_cluster_id(command_params)
        if base_id_required:
            self.executor.add_base_id(command_params)
        if base_cred_required:
            command_params += self.executor.connection.get_base_credentials()
        return super(InfobaseMod, self).execute_command(command_params)

    @required_cluster_id
    def get_base_list(self):
        logger.debug(f'Получаю список баз по соединению {self.executor.connection}')

        params = ['summary', 'list']

        return self.execute_command(params, base_id_required=False)

    @required_cluster_id
    def get_base_by_ref(self, base_ref: str) -> Dict[str, str]:
        logger.debug(f'Ищу базу по имени {base_ref} по соединению {self.executor.connection}')

        base_list = self.get_base_list()
        try:
            base_data = next(filter(lambda x: x.get('name') == base_ref, base_list))
        except StopIteration:
            raise ValueError(f'Нет базы с ref {base_ref}')
        return base_data

    @required_cluster_id
    @required_base_id
    def deny_sessions(self, permission_code: Optional[str] = None):
        logger.debug(f'Запрещаю соединение с базой {self.executor.base_id} по соединению {self.executor.connection}')

        params = ['update', '--sessions-deny=on']
        if permission_code:
            params.append(f'--permission-code={permission_code}')
        self.execute_command(params, base_cred_required=True)

    @required_cluster_id
    @required_base_id
    def allow_sessions(self):
        logger.debug(f'Разрешаю соединение с базой {self.executor.base_id} по соединению {self.executor.connection}')

        params = ['update', '--sessions-deny=off']
        self.execute_command(params, base_cred_required=True)

    @required_cluster_id
    @required_base_id
    def deny_scheduled_jobs(self):
        logger.debug(f'Запрещаю работу регл. задач в базе {self.executor.base_id} '
                     f'по соединению {self.executor.connection}')

        params = ['update', '--scheduled-jobs-deny=on']
        self.execute_command(params, base_cred_required=True)

    @required_cluster_id
    @required_base_id
    def allow_scheduled_jobs(self):
        logger.debug(f'Разрешаю работу регл. задач в базе {self.executor.base_id} '
                     f'по соединению {self.executor.connection}')

        params = ['update', '--scheduled-jobs-deny=off']
        self.execute_command(params, base_cred_required=True)

    @required_cluster_id
    @required_base_id
    def drop_base(self):
        logger.debug(f'Удаляю базу {self.executor.base_id} '
                     f'по соединению {self.executor.connection}')

        params = ['drop']
        self.execute_command(command_params=params, base_cred_required=True)

    @required_cluster_id
    def create_base(self, database_name: str, sql_connection: SqlServerConnection, sql_base_name: str = None):
        logger.debug(f'Создаю базу {self.executor.base_id} '
                     f'по соединению {self.executor.connection}')

        change_timeout = False
        curr_timeout = self.executor.command_timeout
        if curr_timeout < 100:
            change_timeout = True
            self.executor.command_timeout = 100

        _sql_base = sql_base_name if sql_base_name else database_name

        params = [
            'create',
            '--create-database',
            f'--name={database_name}',
            '--license-distribution=allow',
            f'--dbms={sql_connection.type.value}',
            '--locale=ru_RU',
            '--date-offset=2000',
            f'--db-server={sql_connection.server_addr}',
            f'--db-name={_sql_base}',
            f'--db-user={sql_connection.user}',
            f'--db-pwd={sql_connection.password}'
        ]

        new_base = self.execute_command(command_params=params, base_id_required=False)
        if len(new_base) == 0:
            raise SyntaxError(f'Не удалось создать базыу {database_name}')
        self.executor.base_id = new_base[0].get('infobase', None)

        if change_timeout:
            self.executor.command_timeout = curr_timeout

        return self.executor.base_id


class ClusterMod(ABCRacMod):

    def __init__(self, executor: Rac):
        super(ClusterMod, self).__init__(executor=executor, mod='cluster')

    def execute_command(self, command_params: list, cluster_id_required: bool = True) -> List[Dict[str, str]]:
        if cluster_id_required:
            self.executor.add_cluster_id(command_params)
        return super(ClusterMod, self).execute_command(command_params)

    def get_cluster_list(self):
        logger.debug(f'Получаю список кластеров по соединению {self.executor.connection}')

        params = ['list']
        return self.execute_command(params, cluster_id_required=False)

    def get_cluster_data(self, cluster_id: str = None) -> dict:

        if cluster_id:
            params = ['info']
            self.executor.add_cluster_id(params, cluster_id)
            cluster_list = self.execute_command(params, cluster_id_required=False)
        else:
            cluster_list = self.get_cluster_list()

        if not cluster_list:
            raise ValueError(f'Не обнаруженно ни одного кластера!')

        return cluster_list[0]


class SessionMod(ABCRacMod):

    def __init__(self, executor: Rac):
        super(SessionMod, self).__init__(executor=executor, mod='session')

    def execute_command(self, command_params: list,
                        cluster_id_required: bool = True) -> List[Dict[str, str]]:
        if cluster_id_required:
            self.executor.add_cluster_id(command_params)
        return super(SessionMod, self).execute_command(command_params)

    @required_cluster_id
    def get_session_list(self, base_id: Optional[str] = None):
        if not base_id:
            base_id = self.executor.base_id
        logger.debug(f'Получаю список активных сессий в базе {base_id}')

        params = ['list']

        self.executor.add_base_id(params, base_id)

        return self.execute_command(params)

    @required_cluster_id
    def session_info(self, session_id: str):
        logger.debug(f'Получаю информацию о сессии {session_id}')

        params = ['info', f'--session={session_id}']

        return self.execute_command(params)

    @required_cluster_id
    def terminate_session(self, session_id: str, msg: Optional[str] = None):
        logger.debug(f'Удаление сеанса по id {session_id}')

        params = ['terminate', f'--session={session_id}']

        if msg:
            params.append(f'--error-message={msg}')

        return self.execute_command(params)



from designer_cmd.api import RepositoryConnection, Connection, Enterprise, Designer, Rac, RacConnection
from designer_cmd.api.rac_executable import SessionMod, InfobaseMod, ClusterMod, SqlServerConnection, SqlServerType
from typing import List, Dict
import unittest
import os.path as path
import os
from designer_cmd.utils.utils import clear_folder
from json import dump
import socket
import time


class TestConnection(unittest.TestCase):

    def setUp(self):
        pass

    def test_init_base_path(self):
        with self.assertRaises(AttributeError) as cm:
            con = Connection(user='user')
        the_exception = cm.exception
        self.assertEqual(
            the_exception.args[0],
            'Для соедеинения не определен путь к базе!',
            'Полученно неожиданное исключение'
        )

    def test_get_connection_params_file(self):

        loc_path = 'path'

        con = Connection(user='user', password='pass', file_path=loc_path)
        self.assertEqual(
            con.get_connection_params(),
            [f'/F', f'{path.abspath(loc_path)}', '/N', f'user', '/P', f'pass'],
            'Не верно определена строка подключени для файловой базы'
        )

    def test_get_connection_params_server(self):

        con = Connection(user='user', password='pass', server_path='192.168.1.1', server_base_ref='test')
        self.assertEqual(
            con.get_connection_params(),
            ['/S', '192.168.1.1\\test', '/N', 'user', '/P', 'pass'],
            'Не верно определена строка подключени для серверной базы'
        )

    def test_get_connection_string_file(self):

        loc_path = 'path'

        con = Connection(user='user', password='pass', file_path=loc_path)
        self.assertEqual(
            con.get_connection_string(),
            f'Usr=user Pwd=pass File={path.abspath(loc_path)}',
            'Не верно определена строка подключени для файловой базы'
        )


class DataPrepearer(unittest.TestCase):

    def setUp(self):
        test_data_dir = path.join(path.dirname(__file__), 'test_data')

        self.test_data_dir = test_data_dir

        self.temp_path = path.join(test_data_dir, 'temp')
        self.test_base_path = path.join(test_data_dir, 'base')
        self.cf_path = path.join(test_data_dir, '1Cv8.cf')
        self.dt_path = path.join(test_data_dir, '1Cv8.dt')
        self.cf_increment_path = path.join(test_data_dir, '1Cv8_increment.cf')

        self.cfe_path1 = path.join(test_data_dir, 'test.cfe')
        self.cfe_path2 = path.join(test_data_dir, 'test2.cfe')

        self.repo_obj_list = path.join(test_data_dir, 'obj_list')
        self.repo_obj_list_all = path.join(test_data_dir, 'obj_list_all_objects')
        self.merge_settings = path.join(test_data_dir, 'MergeSettings.xml')

        self.repo_user = 'user'
        self.repo_password = 'password'

        self.conn = self.db_connection()
        self.designer = Designer('', self.conn)

        clear_folder(self.test_base_path)
        clear_folder(self.temp_path)

    def tearDown(self):
        clear_folder(self.test_base_path)
        clear_folder(self.temp_path)

    def db_connection(self):
        return Connection(file_path=self.test_base_path)

    def prepare_base(self):
        self.designer.create_base()
        self.designer.load_config_from_file(self.cf_path)
        self.designer.update_db_config()


class TestEnterprise(DataPrepearer):

    def setUp(self) -> None:
        super().setUp()
        self.enterprise = Enterprise('', self.conn)
        self.prepare_base()
        self.epf_path = path.join(self.test_data_dir, 'test_epf.epf')

    def test_execute_epf(self):
        file_path = path.join(self.temp_path, 'test.txt')
        text_data = 'test1'
        json_file = path.join(self.temp_path, 'test.json')
        with open(json_file, r'w') as f:
            dump({'text': text_data, 'file': file_path}, f)

        self.enterprise.run_app(
            ep_x_path=self.epf_path,
            c_string=json_file
        )
        self.assertTrue(path.exists(file_path), 'Файл не создан!')
        with open(file_path, r'r', encoding='utf-8-sig') as f:
            lines = f.readlines()
            self.assertEqual(text_data, lines[0], 'Текст не соответствует ожиданию.')

    def test_run_test_client(self):
        port = 1578
        self.enterprise.run_app(mode=self.enterprise.RunMode.CLIENT, port=port, wait=False)
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            self.assertTrue(s.connect_ex(('localhost', port)) == 0, 'Клиент запущен на неожиданном порту')

    def test_run_test_manager(self):
        self.enterprise.run_app(mode=self.enterprise.RunMode.MANAGER, wait=False)

    def tearDown(self):
        self.enterprise.kill_all_clients()
        time.sleep(0.5)
        super(TestEnterprise, self).tearDown()


class TestDesigner(DataPrepearer):

    def prepare_repo(self) -> str:
        self.designer.create_base()
        repo_path = path.join(self.temp_path, 'repo')
        if not path.exists(repo_path):
            os.mkdir(repo_path)
        clear_folder(repo_path)
        self.designer.repo_connection = RepositoryConnection(repo_path, self.repo_user, self.repo_password)
        self.designer.create_repository()

        return repo_path

    def test_load_create_db(self):
        self.designer.create_base()

    def test_load_conf_from_file(self):
        self.prepare_base()

    def test_update_cfg(self):
        self.designer.create_base()
        self.designer.load_config_from_file(self.cf_path)
        self.designer.update_db_config()

    def test_dump_conf_to_file(self):
        self.prepare_base()
        cf_file_path = path.join(self.temp_path, '1Cv81.cf')
        self.designer.dump_config_to_file(cf_file_path)

        self.assertTrue(os.path.exists(cf_file_path), 'Выгрузка кофигурации не создана!')

    def test_dump_config_to_files(self):
        self.prepare_base()
        dir_xml_config_path = self.dump_to_files()

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'Не обнаружена выгрузка в xml'
        )

    def test_dump_config_to_file_increment(self):
        self.prepare_base()

        dir_xml_config_path = self.dump_to_files()

        cur_catalog_file_time = os.path.getatime(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml'))

        self.designer.load_config_from_file(self.cf_path)
        self.designer.update_db_config()
        self.designer.dump_config_to_files(dir_xml_config_path)

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник2.xml')),
            'Выгрузка не была выполнена.'
        )

        self.assertEqual(
            cur_catalog_file_time,
            os.path.getatime(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'Выгрузка не была инкриментальной'
        )

    def test_dump_extension_to_file(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

        cf_file_path = path.join(self.temp_path, 'test_.cfe')
        self.designer.dump_extension_to_file(cf_file_path, 'test')

        self.assertTrue(
            os.path.exists(cf_file_path),
            'Не обнаружена выгрузка расширения'
        )

    def export_all_extension(self):
        cfe_dir_path = path.join(self.temp_path, 'cfe_dir')
        if not path.exists(cfe_dir_path):
            os.mkdir(cfe_dir_path)
        clear_folder(cfe_dir_path)

        self.designer.dump_extensions_to_files(cfe_dir_path)

        return cfe_dir_path

    def test_dump_extensions_to_files(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

        cfe_dir_path = self.export_all_extension()

        self.assertTrue(
            os.path.exists(os.path.join(cfe_dir_path, 'test')),
            'Выгрузка не сформирована.'
        )

        self.assertTrue(
            os.path.exists(os.path.join(cfe_dir_path, 'test2')),
            'Выгрузка не сформирована.'
        )

    def test_load_extension(self):
        self.prepare_base()
        self.designer.load_extension_from_file(self.cfe_path1, 'test')
        self.designer.load_extension_from_file(self.cfe_path2, 'test2')

        cfe_dir_path = self.export_all_extension()

        self.assertTrue(
            os.path.exists(os.path.join(cfe_dir_path, 'test')),
            'Отсутствует расширение test.'
        )

        self.assertTrue(
            os.path.exists(os.path.join(cfe_dir_path, 'test2')),
            'Отсутствует расширение test2.'
        )

    def test_load_extension_from_files(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

        cfe_dir_path = path.join(self.temp_path, 'cfe_dir', 'test')
        self.designer.dump_extension_to_files(cfe_dir_path, 'test')

        self.assertTrue(
            os.path.exists(cfe_dir_path),
            'Выгрузка не сформирована.'
        )

        self.designer.load_extension_from_files(cfe_dir_path, 'test')

    def test_delete_extension(self):
        self.prepare_base()
        ext_name = 'test'

        self.designer.load_extension_from_file(self.cfe_path1, ext_name)

        self.designer.delete_extension(ext_name)

        with self.assertRaises(SyntaxError) as ex:
            self.designer.delete_extension(ext_name)

        self.assertIn(
            'расширение конфигурации с указанным именем не найдено: test',
            ex.exception.args[0],
            'Ошибка не включает в себя текст ошибки поиска расширения'
        )

    def test_check_extension(self):
        self.prepare_base()
        ext_name = 'test'

        self.designer.load_extension_from_file(self.cfe_path1, ext_name)
        self.designer.check_apply_extension(ext_name)

    def test_add_repository(self):
        repo_path = self.prepare_repo()
        self.assertTrue(os.path.exists(os.path.join(repo_path, 'cache')), 'Хранилище не создано.')

    def test_add_user_to_repo(self):
        self.prepare_repo()
        self.designer.add_user_to_repository('user1', 'password1', rights='LockObjects')

    def test_lock_obj_in_repository(self):
        self.prepare_repo()
        self.designer.lock_objects_in_repository(self.repo_obj_list_all, True)

    def test_unlock_obj_in_repository(self):
        self.prepare_repo()
        self.designer.unlock_objects_in_repository(self.repo_obj_list_all)

    def test_commit_repo(self):

        comment = 'test_test_test_1_2_3_4_5'

        self.prepare_repo()
        self.designer.lock_objects_in_repository(self.repo_obj_list_all, True)
        self.designer.merge_config_with_file(self.cf_increment_path, self.merge_settings)
        self.designer.update_db_config()
        self.designer.commit_config_to_repo(comment, self.repo_obj_list_all)

        report_path = path.join(self.temp_path, 'repo_report.xml')
        self.designer.get_repo_report(report_path)

        with open(report_path, 'r', encoding='utf-8') as f:
            data = f.read()
            self.assertTrue(
                comment in data,
                'Commit не создан.'
            )

    def test_dump_config_from_repo(self):
        self.test_commit_repo()
        cf_file_path = path.join(self.temp_path, '1Cv81_repo.cf')
        self.designer.dump_config_to_file_from_repo(cf_file_path)
        self.assertTrue(os.path.exists(cf_file_path), 'Выгрузка кофигурации не создана!')

        self.check_common_module()

    def test_update_conf_from_repo(self):
        self.test_commit_repo()
        self.designer.load_db_from_file(self.dt_path)
        self.designer.update_conf_from_repo()
        self.check_common_module()

    def test_get_repo_report(self):
        self.prepare_repo()

        report_path = path.join(self.temp_path, 'repo_report.xml')
        self.designer.get_repo_report(report_path)

        self.assertTrue(
            os.path.exists(report_path),
            'Отчет не сформирован.'
        )

        with open(report_path, 'r', encoding='utf-8') as f:
            data = f.read()
            self.assertTrue(
                'Создание хранилища конфигурации' in data,
                'В отчете нет необходимых данных'
            )

    def test_encoding(self):
        self.designer.create_base()
        with self.assertRaises(SyntaxError) as ex:
            self.designer.create_base()
        self.assertIn('Указанная информационная база уже существует.', ex.exception.msg, 'Небыло вызвано исключение')

    def test_bind_cfg_to_repo(self):
        self.prepare_repo()
        self.designer.load_db_from_file(self.dt_path)

        self.designer.bind_cfg_to_repo()
        dir_xml_config_path = self.dump_to_files()

        self.assertFalse(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'База не была привязана.'
        )

    def test_merge_cf(self):
        self.prepare_base()
        self.designer.merge_config_with_file(self.cf_increment_path, self.merge_settings)

    def test_compare_conf(self):
        self.prepare_base()
        report_path = os.path.join(self.temp_path, 'report.txt')
        self.designer.compare_config_with_file(self.cf_path, report_path)

        self.assertTrue(os.path.exists(report_path), 'Отчет о сравнении не создан!')

    def test_dump_db_to_file(self):
        self.prepare_base()
        new_dt_path = os.path.join(self.temp_path, '1c.dt')
        self.designer.dump_db_to_file(new_dt_path)

        self.assertTrue(os.path.exists(new_dt_path), 'Файл выгрузки не создан!')

    def test_load_db_from_file(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

        dir_xml_config_path = self.dump_to_files()

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'Выгрузка не была выполнена.'
        )

    def test_unbind_cfg_from_repo(self):
        self.prepare_repo()
        new_user = 'new_user'
        new_passwd = 'passwd'
        self.designer.add_user_to_repository(new_user, new_passwd, 'LockObjects')

        self.designer.unbind_cfg_from_repo(True, True)

        self.designer.repo_connection.user = new_user
        self.designer.repo_connection.password = new_passwd

        self.designer.bind_cfg_to_repo()

        self.designer.repo_connection.user = self.repo_user
        self.designer.repo_connection.password = self.repo_password

        with self.assertRaises(SyntaxError) as er:
            self.designer.lock_objects_in_repository(self.repo_obj_list_all, True)
        self.assertIn('Ошибка захвата объектов в хранилище', er.exception.msg,
                      'Ошибка захвата объектов не была выброшена.')

        self.designer.repo_connection.user = new_user
        self.designer.repo_connection.password = new_passwd

        self.designer.lock_objects_in_repository(self.repo_obj_list_all, True)

    def check_common_module(self):
        dir_xml_config_path = self.dump_to_files()

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'CommonModules\\ОбщийМодуль1.xml')),
            'Объединение не выполнено.'
        )

    def dump_to_files(self) -> str:
        dir_xml_config_path = path.join(self.temp_path, 'xml_config')
        if not path.exists(dir_xml_config_path):
            os.mkdir(dir_xml_config_path)

        self.designer.dump_config_to_files(dir_xml_config_path)
        return dir_xml_config_path


class ExecutorMock(Rac):

    def __init__(self):
        conn = RacConnection()
        super(ExecutorMock, self).__init__('', conn)
        self.params = []
        self.mode = ''
        self.test_data = []

    def execute_command(self, mode: str, command_params: list,
                        credentials_required: bool = False) -> List[Dict[str, str]]:
        self.params = command_params
        self.mode = mode
        return self.test_data


class TestInfobaseMod(unittest.TestCase):

    def setUp(self) -> None:
        self.mock = ExecutorMock()
        self.mod = InfobaseMod(executor=self.mock)
        self.cluster_id = 'ff'
        self.mock.set_cluster_id(self.cluster_id)

    def test_get_infobase_list(self):
        self.mod.get_base_list()

        self.assertEqual({'summary', 'list', f'--cluster={self.cluster_id}'}, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')
        self.assertEqual('infobase', self.mock.mode, 'Режим не соответствует ожидаемому')

    def test_get_base_by_ref(self):
        data1 = {'name': '2'}
        self.mock.test_data = [{'name': '1'}, data1, {'name': '3'}]
        data = self.mod.get_base_by_ref('2')
        self.assertEqual(data, data1, 'Получен неожиданные результат поиска базы')

    def test_deny_sessions(self):
        base_id = 'bb'
        permission_code = '111'
        self.mock.base_id = base_id
        self.mod.deny_sessions(permission_code)
        self.assertEqual({'update', f'--cluster={self.cluster_id}', f'--infobase={base_id}',
                          '--sessions-deny=on', f'--permission-code={permission_code}'}, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')
        self.assertEqual('infobase', self.mock.mode, 'Режим не соответствует ожидаемому')

        self.mod.deny_sessions()
        self.assertEqual({'update', f'--cluster={self.cluster_id}', f'--infobase={base_id}',
                          '--sessions-deny=on'}, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')
        self.assertEqual('infobase', self.mock.mode, 'Режим не соответствует ожидаемому')

    def test_allow_sessions(self):
        base_id = 'bb'
        self.mock.base_id = base_id
        self.mod.allow_sessions()
        self.assertEqual({'update', f'--cluster={self.cluster_id}', f'--infobase={base_id}',
                          '--sessions-deny=off'}, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')
        self.assertEqual('infobase', self.mock.mode, 'Режим не соответствует ожидаемому')

    def test_deny_sheduled_jobs(self):
        base_id = 'bb'
        self.mock.base_id = base_id
        self.mod.deny_scheduled_jobs()
        self.assertEqual({'update', f'--cluster={self.cluster_id}', f'--infobase={base_id}',
                          '--scheduled-jobs-deny=on'}, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')
        self.assertEqual('infobase', self.mock.mode, 'Режим не соответствует ожидаемому')

    def test_allow_sheduled_jobs(self):
        base_id = 'bb'
        self.mock.base_id = base_id
        self.mod.allow_scheduled_jobs()
        self.assertEqual({'update', f'--cluster={self.cluster_id}', f'--infobase={base_id}',
                          '--scheduled-jobs-deny=off'}, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')
        self.assertEqual('infobase', self.mock.mode, 'Режим не соответствует ожидаемому')

    def test_drop_database(self):
        base_id = 'bb'
        self.mock.base_id = base_id
        self.mod.drop_base()

        self.assertEqual('infobase', self.mock.mode, 'Режим не соответствует ожидаемому')

        self.assertEqual({'drop', f'--cluster={self.cluster_id}', f'--infobase={base_id}'}, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')

    def test_create_data_base(self):
        db_name = 'bb'
        host = 'host'
        user = 'user'
        password = 'pwd'
        server_type = SqlServerType.MSSQL
        base_id = '1'

        self.mock.test_data = [{'infobase': base_id}]

        sql_conn = SqlServerConnection(host=host, user=user, password=password, type=server_type)
        new_base_id = self.mod.create_base(db_name, sql_conn)

        self.assertEqual('infobase', self.mock.mode, 'Режим не соответствует ожидаемому')

        self.assertEqual({
            'create',
            '--cluster=ff',
            '--create-database',
            f'--name={db_name}',
            '--license-distribution=allow',
            f'--dbms=MSSQLServer',
            '--locale=ru_RU',
            '--date-offset=2000',
            f'--db-server={host}:5432',
            f'--db-name={db_name}',
            f'--db-user={user}',
            f'--db-pwd={password}'
        }, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')

        self.assertEqual(new_base_id, base_id, 'Вернувшийся результат не равен ожидаемому!')


class TestClusterMod(unittest.TestCase):

    def setUp(self) -> None:
        self.mock = ExecutorMock()
        self.mod = ClusterMod(executor=self.mock)
        self.cluster_id = 'ff'
        self.mock.set_cluster_id(self.cluster_id)

    def test_get_cluster_list(self):
        self.mod.get_cluster_list()

        self.assertEqual({'list'}, set(self.mock.params),
                         'Сформированная команда не соответствует ожидаемой')
        self.assertEqual('cluster', self.mock.mode, 'Режим не соответствует ожидаемому')

    def test_get_base_by_ref(self):
        data1 = {'name': '2'}
        self.mock.test_data = [data1, {'name': '1'}, {'name': '3'}]

        data = self.mod.get_cluster_data()

        self.assertEqual(data, data1, 'Получен неожиданные результат поиска кластера')

        self.mod.get_cluster_data(self.cluster_id)

        self.assertEqual({'info', f'--cluster={self.cluster_id}'},
                         set(self.mock.params), 'Сформированная команда на соответствует ожидаемой.')
        self.assertEqual('cluster', self.mock.mode, 'Режим не соответствует ожидаемому')


class TestSessionMod(unittest.TestCase):

    def setUp(self) -> None:
        self.mock = ExecutorMock()
        self.mod = SessionMod(executor=self.mock)

        self.cluster_id = 'ff'
        self.mock.set_cluster_id(self.cluster_id)

        self.base_id = 'bb'
        self.mock.base_id = self.base_id

        self.session_id = 'ss'

    def test_get_session_list(self):

        self.mod.get_session_list(self.base_id)

        self.assertEqual({'list', f'--cluster={self.cluster_id}', f'--infobase={self.base_id}'},
                         set(self.mock.params), 'Сформированная команда на соответствует ожидаемой.')
        self.assertEqual('session', self.mock.mode, 'Режим не соответствует ожидаемому')

        self.mod.get_session_list()

        self.assertEqual({'list', f'--cluster={self.cluster_id}', f'--infobase={self.base_id}'},
                         set(self.mock.params), 'Сформированная команда на соответствует ожидаемой.')
        self.assertEqual('session', self.mock.mode, 'Режим не соответствует ожидаемому')

    def test_session_info(self):
        self.mod.session_info(self.session_id)

        self.assertEqual({'info', f'--cluster={self.cluster_id}', f'--session={self.session_id}'},
                         set(self.mock.params), 'Сформированная команда на соответствует ожидаемой.')
        self.assertEqual('session', self.mock.mode, 'Режим не соответствует ожидаемому')

    def terminate_session(self):
        msg = 'Удалено админом'
        self.mod.terminate_session(self.session_id, msg)

        self.assertEqual({'terminate', f'--cluster={self.cluster_id}',
                          f'--session={self.session_id}', f'--error-message={msg}'},
                         set(self.mock.params), 'Сформированная команда на соответствует ожидаемой.')
        self.assertEqual('session', self.mock.mode, 'Режим не соответствует ожидаемому')



from .main_executable import (Enterprise, Connection, RepositoryConnection, Designer, convert_cf_to_xml,
                              convert_cfe_to_xml, xml_conf_version_file_exists)
from .rac_executable import Rac, RacConnection, SqlServerType, SqlServerConnection

__all__ = [
    'Enterprise',
    'Connection',
    'RepositoryConnection',
    'Designer',
    'convert_cfe_to_xml',
    'convert_cf_to_xml',
    'xml_conf_version_file_exists',
    'Rac',
    'RacConnection',
    'SqlServerType',
    'SqlServerConnection',
]



from .utils import (get_1c_exe_path, get_rac_path, execute_command, xml_conf_version_file_exists,
                    PlatformVersion, clear_folder, windows_platform, port_in_use, get_1c_processes, kill_process)
